@@ -732,14 +732,20 @@ e_completion_view_construct (ECompletionView *cv, ECompletion *completion)
 
 	cv->model = e_table_simple_new (table_col_count,
 					table_row_count,
+					NULL,
+
 					table_value_at,
 					NULL,
 					table_is_cell_editable,
+
+					NULL, NULL,
+
 					NULL, NULL, NULL, NULL,
 					table_value_to_string,
 					cv);
 
 	cv->table = e_table_scrolled_new (cv->model, NULL, simple_spec, NULL);
+	gtk_object_unref (GTK_OBJECT (cv->model));
 
 	e_scroll_frame_set_shadow_type (E_SCROLL_FRAME (cv->table), GTK_SHADOW_NONE);
 	e_scroll_frame_set_scrollbar_spacing (E_SCROLL_FRAME (cv->table), 0);
