@@ -471,8 +471,9 @@ e_text_destroy (GtkObject *object)
 	if (text->invisible)
 		gtk_object_unref (GTK_OBJECT(text->invisible));
 
-	if (text->lines)
-		g_free (text->lines);
+	g_free (text->lines);
+	g_free (text->primary_selection);
+	g_free (text->clipboard_selection);
 
 	if (text->font)
 		e_font_unref (text->font);
@@ -519,9 +520,16 @@ e_text_destroy (GtkObject *object)
 static void
 e_text_text_model_changed (ETextModel *model, EText *text)
 {
+	gint model_len = e_text_model_get_text_length (model);
 	text->text = e_text_model_get_text(model);
 	e_text_free_lines(text);
+
 	gtk_signal_emit (GTK_OBJECT (text), e_text_signals[E_TEXT_CHANGED]);
+
+	/* Make sure our selection doesn't extend past the bounds of our text. */
+	text->selection_start = CLAMP (text->selection_start, 0, model_len);
+	text->selection_end   = CLAMP (text->selection_end,   0, model_len);
+
 	text->needs_split_into_lines = 1;
 	e_canvas_item_request_reflow (GNOME_CANVAS_ITEM(text));
 }
@@ -3721,15 +3729,11 @@ e_text_supply_selection (EText *text, guint time, GdkAtom selection, guchar *dat
 	invisible = e_text_get_invisible(text);
 
 	if (selection == GDK_SELECTION_PRIMARY ) {
-		if (text->primary_selection) {
-			g_free (text->primary_selection);
-		}
+		g_free (text->primary_selection);
 		text->primary_selection = g_strndup(data, length);
 		text->primary_length = length;
 	} else if (selection == clipboard_atom) {
-		if (text->clipboard_selection) {
-			g_free (text->clipboard_selection);
-		}
+		g_free (text->clipboard_selection);
 		text->clipboard_selection = g_strndup(data, length);
 		text->clipboard_length = length;
 	}
