@@ -1555,3 +1555,23 @@ e_group_bar_stop_all_animation	(EGroupBar	*group_bar)
 	}
 }
 
+
+/*
+ * Get the group button.
+ */
+GtkWidget*
+e_group_bar_get_group_button		(EGroupBar	*group_bar,
+					 gint		 group_num)
+{
+	EGroupBarChild *group;
+
+	g_return_val_if_fail (E_IS_GROUP_BAR (group_bar), NULL);
+	g_return_val_if_fail (group_num >= 0, NULL);
+	g_return_val_if_fail (group_num < group_bar->children->len, NULL);
+
+	group = &g_array_index (group_bar->children, EGroupBarChild,
+				group_num);
+
+	return group->button;
+}
+
