@@ -14,6 +14,7 @@
 
 #include <gal/util/e-i18n.h>
 #include <gal/util/e-util.h>
+#include <gal/widgets/e-unicode.h>
 #include <gtk/gtksignal.h>
 
 struct ECategoriesMasterListComboPriv {
@@ -38,12 +39,17 @@ ecmlc_ecml_changed (ECategoriesMasterList *ecml, ECategoriesMasterListCombo *ecm
 	int i;
 	GList *strings = NULL;
 	for (i = 0; i < count; i++) {
-		const char *category = e_categories_master_list_nth (ecml, i);
-		strings = g_list_prepend (strings, (gpointer) category);
+		char *category;
+
+		category = e_utf8_to_gtk_string (
+			GTK_WIDGET (ecmlc),
+			e_categories_master_list_nth (ecml, i));
+		strings = g_list_prepend (strings, category);
 	}
 	strings = g_list_reverse (strings);
-	strings = g_list_prepend (strings, "");
+	strings = g_list_prepend (strings, g_strdup (""));
 	gtk_combo_set_popdown_strings (GTK_COMBO (ecmlc), strings);
+	g_list_foreach (strings, (GFunc) g_free, NULL);
 	g_list_free (strings);
 }
 
