@@ -135,6 +135,12 @@ dialog_destroyed (GtkObject *dialog, ECategoriesMasterListDialog *ecmld)
 	gtk_object_destroy (GTK_OBJECT (ecmld));
 }
 
+static void
+dialog_clicked (GtkObject *dialog, int button, ECategoriesMasterListDialog *ecmld)
+{
+	gnome_dialog_close (GNOME_DIALOG (dialog));
+}
+
 #if 0
 static void
 connect_button (ECategoriesMasterListDialog *ecmld, GladeXML *gui, const char *widget_name, void *cback)
@@ -193,6 +199,9 @@ setup_gui (ECategoriesMasterListDialog *ecmld)
 
 	gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
 			    GTK_SIGNAL_FUNC (dialog_destroyed), ecmld);
+	if (dialog && GNOME_IS_DIALOG (dialog))
+		gtk_signal_connect (GTK_OBJECT (dialog), "clicked",
+				    GTK_SIGNAL_FUNC (dialog_clicked), ecmld);
 }
 
 static void
