@@ -98,6 +98,7 @@ e_font_from_gdk_name (const gchar *name)
 		need_fontset = TRUE;
 		g_snprintf (n, 1024, name);
 	} else {
+#if 0
 		gchar *c[14], *nn;
 		gint i;
 		/* Split name into pieces */
@@ -117,11 +118,14 @@ e_font_from_gdk_name (const gchar *name)
 			/* fixme: Whoever actually uses CJK locale should check/change that */
 			if (!strncasecmp ("eucjp", c[E_XF_CHARSET], len)) need_fontset = TRUE;
 			else if (!strncasecmp ("ujis", c[E_XF_CHARSET], len)) need_fontset = TRUE;
-			else if (!strncasecmp ("jisx208", c[E_XF_CHARSET], len)) need_fontset = TRUE;
+			else if (!strncasecmp ("jis", c[E_XF_CHARSET], 3)) need_fontset = TRUE;
 			else if (!strncasecmp ("gb2312", c[E_XF_CHARSET], len)) need_fontset = TRUE;
 			else if (!strncasecmp ("ksc5601", c[E_XF_CHARSET], len)) need_fontset = TRUE;
 		}
 		g_free (nn);
+#else
+		if (MB_CUR_MAX > 1) need_fontset = TRUE;
+#endif
 		if (need_fontset) {
 			/* We do not have ',', so we have to extend name */
 			g_snprintf (n, 1024, "%s,*", name);
@@ -212,6 +216,7 @@ e_font_from_gdk_font (GdkFont *gdkfont)
 	}
 
 	if (gdkfont->type == GDK_FONT_FONTSET) {
+#if 0
 		gchar *xlocale[] = {	"ja_JP.euc", "ja_JP.ujis",
 					"zh_CN.GB2312", "zh_TW.Big5",
 					"ko_KR.eucKR", NULL};
@@ -232,6 +237,18 @@ e_font_from_gdk_font (GdkFont *gdkfont)
 				return font;
 			}
 		}
+#else
+		if (MB_CUR_MAX > 1) {
+			/* We need fontset */
+			font = e_font_from_gdk_fontset (gdkfont);
+			/* Insert into cache */
+			if (e_cache_insert (cache, keyfont, font, 1)) {
+				/* Inserted, so add cache ref manually */
+				e_font_ref (font);
+			}
+			return font;
+		}
+#endif
 	}
 
 	/* We can do with simple font */
@@ -762,7 +779,7 @@ e_font_draw_utf8_text (GdkDrawable *drawable, EFont *font, EFontStyle style, Gdk
 	native = alloca (numbytes * 4);
 
 	native_bytes = e_font_to_native (font, native, text, numbytes);
-
+	
 	if ((style & E_FONT_BOLD) && (font->bold)) {
 		gdk_draw_text (drawable, font->bold, gc, x, y, native, native_bytes);
 	} else {
@@ -853,15 +870,34 @@ translate_encoding (const gchar *encoding)
 		g_hash_table_insert (eh, "iso_8859-14", "iso-8859-14");
 		g_hash_table_insert (eh, "iso8859-15", "iso-8859-15");
 		g_hash_table_insert (eh, "iso_8859-15", "iso-8859-15");
+
+		/* fixme: should the following be iso-10646/ucs2? */
 		g_hash_table_insert (eh, "iso10646-1", "iso-10646-1");
 		g_hash_table_insert (eh, "iso_10646-1", "iso-10646-1");
+
 		g_hash_table_insert (eh, "koi8-r", "koi8-r");
 		g_hash_table_insert (eh, "koi8-ub", "koi8-ub");
-		g_hash_table_insert (eh, "eucjp", "EUC-JP");
-		g_hash_table_insert (eh, "ujis", "EUC-JP");
+
+		/* fixme: Japanese and Korean users, please fix the following: */
+		g_hash_table_insert (eh, "eucjp-0", "EUC-JP");
+		g_hash_table_insert (eh, "euc-jp", "EUC-JP");
+		g_hash_table_insert (eh, "ujis-0", "ujis");
+		g_hash_table_insert (eh, "jisx0208.1983-0", "shift-jis");
+		g_hash_table_insert (eh, "jisx0212.1990-0", "shift-jis");
+		g_hash_table_insert (eh, "euckr-0", "euc-kr");
+		g_hash_table_insert (eh, "euc-kr", "euc-kr");
+
+		g_hash_table_insert (eh, "big5-0", "big5");
+		g_hash_table_insert (eh, "big5.eten-0", "big5");
+		g_hash_table_insert (eh, "big5hkscs-0", "big5hkscs");
+		g_hash_table_insert (eh, "gb2312-0", "gb2312");
+		g_hash_table_insert (eh, "gb2312.1980-0", "gb2312");
+		g_hash_table_insert (eh, "gb18030-0", "gb18030");
+		g_hash_table_insert (eh, "gbk-0", "gbk");
 	}
 
 	strncpy (e, encoding, 64);
+	e[63] = '\0';
 	g_strdown (e);
 
 	finalenc = g_hash_table_lookup (eh, e);
@@ -878,19 +914,29 @@ get_locale_charset (void)
 
 	if (!initialized) {
 		locale = gtk_set_locale ();
-		if ( strcmp (locale, "C") == 0 )
+		if (strcmp (locale, "C") == 0)
 			locale = NULL;
-		if ( locale != NULL ) {
+		if (locale != NULL) {
+#ifdef HAVE_CODESET
+			charset = g_strdup(nl_langinfo(CODESET));
+			g_strdown(charset);
+#else
 			gchar * p;
 			size_t len;
 
 			p = strchr (locale, '@');
-			len = p ? (p - locale) : strlen(locale);
-			p = strchr (locale, '.');
 			if (p) {
-				charset = g_strndup(p+1, len - (p - locale) + 1);
+				len = p - locale;
+			} else {
+				p = strchr (locale, ';');
+				len = p ? (p - locale) : strlen (locale);
+			}
+			p = strchr (locale, '.');
+			if (p && p < (locale + len - 1)) {
+				charset = g_strndup (p + 1, len - (p - locale) + 1);
 				g_strdown (charset);
 			}
+#endif
 		}
 		initialized = TRUE;
 	}
@@ -930,7 +976,10 @@ e_locale_encoding (void)
 	const char *encoding = nl_langinfo(CODESET);
 	if (encoding) {
 		const gchar *trans_encoding = translate_encoding (encoding);
-		if (trans_encoding) return trans_encoding;
+		if (trans_encoding)
+			return trans_encoding;
+		else
+			return encoding;
 	}
 #endif
 	return NULL;
@@ -970,7 +1019,10 @@ e_gdk_font_encoding (GdkFont *font)
 		encoding = get_locale_charset ();
 		if (encoding) {
 			const gchar *trans_encoding = translate_encoding (encoding);
-			if (trans_encoding) return trans_encoding;
+			if (trans_encoding)
+			    return trans_encoding;
+			else
+			    return encoding;
 		}
 	}
 
@@ -1021,8 +1073,8 @@ e_gdk_font_encoding (GdkFont *font)
 	return encoding;
 }
 
-static iconv_t
-e_iconv_from_encoding (const gchar *enc)
+iconv_t
+e_iconv_from_charset (const gchar *enc)
 {
 	static GHashTable *ih = NULL;
 	iconv_t ic;
@@ -1042,8 +1094,8 @@ e_iconv_from_encoding (const gchar *enc)
 	return ic;
 }
 
-static iconv_t
-e_iconv_to_encoding (const gchar *enc)
+iconv_t
+e_iconv_to_charset (const gchar *enc)
 {
 	static GHashTable *ih = NULL;
 	iconv_t ic;
@@ -1069,7 +1121,7 @@ e_iconv_from_locale (void)
 	const gchar *enc;
 
 	enc = e_locale_encoding ();
-	return e_iconv_from_encoding (enc);
+	return e_iconv_from_charset (enc);
 }
 
 iconv_t
@@ -1078,7 +1130,7 @@ e_iconv_to_locale (void)
 	const gchar *enc;
 
 	enc = e_locale_encoding ();
-	return e_iconv_to_encoding (enc);
+	return e_iconv_to_charset (enc);
 }
 
 iconv_t
@@ -1090,7 +1142,7 @@ e_iconv_from_gdk_font (GdkFont *font)
 
 	enc = e_gdk_font_encoding (font);
 
-	return e_iconv_from_encoding (enc);
+	return e_iconv_from_charset (enc);
 }
 
 iconv_t
@@ -1102,7 +1154,7 @@ e_iconv_to_gdk_font (GdkFont *font)
 
 	enc = e_gdk_font_encoding (font);
 
-	return e_iconv_to_encoding (enc);
+	return e_iconv_to_charset (enc);
 }
 
 /*
@@ -1272,8 +1324,6 @@ e_font_print_gdk_font_name (const GdkFont * font)
 
 	font_atom = gdk_atom_intern ("FONT", FALSE);
 
-	g_print ("-------- start of font description --------\n");
-
 	if (font == NULL) {
 		g_print ("GdkFont is NULL\n");
 	} else if (font->type == GDK_FONT_FONTSET) {
@@ -1295,7 +1345,5 @@ e_font_print_gdk_font_name (const GdkFont * font)
 		g_print ("GdkFont: %s\n", name);
 		if (name) g_free (name);
 	}
-
-	g_print ("-------- end of font description --------\n");
 }
 #endif
